package com.qmth.sop.business.mapper;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.sop.business.bean.dto.DataPermissionDto;
import com.qmth.sop.business.bean.result.CrmArchivesAllocationResult;
import com.qmth.sop.business.bean.result.SysLogResult;
import com.qmth.sop.business.bean.result.UserArchivesAllocationResult;
import com.qmth.sop.business.entity.TBUserArchivesAllocation;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.qmth.sop.common.enums.CrmStatusEnum;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * <p>
 * 人员分配表 Mapper 接口
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
public interface TBUserArchivesAllocationMapper extends BaseMapper<TBUserArchivesAllocation> {
    /**
     * 查询派单档案分配 派单分页列表
     *
     * @param iPage         分页参数
     * @param serviceUnitId 服务单元id
     * @param province      省
     * @param city          市
     * @param area          县
     * @param customName    客户名称
     * @param gap           差额
     * @param regionId      大区id
     * @param dpr           数据权限
     * @return 分页查询结果
     */
    IPage<UserArchivesAllocationResult> findCrmAllocationPage(@Param("iPage") Page<UserArchivesAllocationResult> iPage,
                                                              @Param("serviceUnitId") Long serviceUnitId,
                                                              @Param("province") String province,
                                                              @Param("city") String city,
                                                              @Param("area") String area,
                                                              @Param("customName") String customName,
                                                              @Param("gap") Integer gap,
                                                              @Param("regionId") Long regionId,
                                                              @Param("dpr") DataPermissionDto dpr);

    /**
     * 查询派单档案分配 派单小计列表
     *
     * @param serviceUnitId 服务单元id
     * @param province      省
     * @param city          市
     * @param area          县
     * @param customName    客户名称
     * @param gap           差额
     * @param crmIdList     派单id集合
     * @param crmStatus     派单状态枚举
     * @param regionId      大区id
     * @param dpr           数据权限
     * @return 查询结果
     */
    List<UserArchivesAllocationResult> findCrmAllocationSubTotal(@Param("serviceUnitId") Long serviceUnitId,
                                                                 @Param("province") String province,
                                                                 @Param("city") String city,
                                                                 @Param("area") String area,
                                                                 @Param("customName") String customName,
                                                                 @Param("gap") Integer gap,
                                                                 @Param("crmIdList") List<Long> crmIdList,
                                                                 @Param("crmStatus") CrmStatusEnum crmStatus,
                                                                 @Param("regionId") Long regionId,
                                                                 @Param("dpr") DataPermissionDto dpr);

    List<CrmArchivesAllocationResult> findAllocationByCrmNo(@Param("crmNo") String crmNo);
}
