package com.qmth.sop.business.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.sop.business.bean.params.DeviceInOutSubmitParam;
import com.qmth.sop.business.bean.result.DeviceInOutResult;
import com.qmth.sop.business.entity.TBDeviceInOut;
import com.qmth.sop.common.enums.DeviceStatusEnum;
import com.qmth.sop.common.enums.DeviceUsageTypeEnum;

import java.util.List;

/**
 * <p>
 * 设备出入库登记 服务类
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
public interface TBDeviceInOutService extends IService<TBDeviceInOut> {
    // -- 设备出入库登记 sop部分

    /**
     * sop管理 - 设备出入库登记查询
     *
     * @param serviceUnitId  服务单元id
     * @param usageType      用途类型
     * @param userId         登记人
     * @param deviceStatus   运行状态
     * @param inOutTimeStart 出入库时间 - 开始
     * @param inOutTimeEnd   出入库时间 - 结束
     * @param deviceNo       设备编号
     * @param deviceSerialNo 设备序列号
     * @param customName     客户名称
     * @param location       当前地
     * @param address        发往地
     * @param serialNo       登记流水号
     * @param pageNumber     分页页数
     * @param pageSize       分页容量
     * @return 分页结果
     */
    IPage<DeviceInOutResult> findDeviceInOutPageBySop(Long serviceUnitId, DeviceUsageTypeEnum usageType, Long userId,
                                                      DeviceStatusEnum deviceStatus, Long inOutTimeStart, Long inOutTimeEnd,
                                                      String deviceNo, String deviceSerialNo, String customName, String location, String address, String serialNo,
                                                      Integer pageNumber, Integer pageSize);

    /**
     * 查询可出库的设备信息
     *
     * @param supplierId 供应商id
     * @param sopNo      sopNo
     * @return 可出库设备信息
     */
    List<DeviceInOutResult> findDeviceCanOutInfo(Long supplierId, String sopNo);

    /**
     * 查询可入库设备信息
     *
     * @param supplierId 供应商id
     * @param sopNo      sopNo
     * @return 可入库设备信息
     */
    List<DeviceInOutResult> findDeviceCanInInfo(Long supplierId, String sopNo);

    /**
     * 设备出入库提交
     *
     * @param deviceInOutSubmitParam 设备出入库提交参数
     */
    void deviceInOutSubmit(DeviceInOutSubmitParam deviceInOutSubmitParam);


    // -- 出入库登记查询 资源保障部分

    /**
     * 资源保障-设备保障-出入库登记查询
     *
     * @param serviceUnitId  服务单元id
     * @param usageType      用途类型
     * @param userId         登记人
     * @param deviceStatus   运行状态
     * @param inOutTimeStart 出入库时间 - 开始
     * @param inOutTimeEnd   出入库时间 - 结束
     * @param deviceNo       设备编号
     * @param deviceSerialNo 设备序列号
     * @param customName     客户名称
     * @param location       当前地
     * @param address        发往地
     * @param serialNo       登记流水号
     * @param pageNumber     分页页数
     * @param pageSize       分页容量
     * @return 分页结果
     */
    IPage<DeviceInOutResult> findDeviceInOutPageBySource(Long serviceUnitId, DeviceUsageTypeEnum usageType, Long userId,
                                                         DeviceStatusEnum deviceStatus, Long inOutTimeStart, Long inOutTimeEnd,
                                                         String deviceNo, String deviceSerialNo, String customName, String location, String address, String serialNo,
                                                         Integer pageNumber, Integer pageSize);
}
